@@ -15,13 +15,9 @@ use VladimirYuldashev\LaravelQueueRabbitMQ\Queue\Jobs\RabbitMQJob;
 
 class RabbitMQQueue extends Queue implements QueueContract
 {
-    /**
-     * Used for retry logic, to set the retries on the message metadata instead of the message body.
-     */
-    const ATTEMPT_COUNT_HEADERS_KEY = 'attempts_count';
-
     protected $sleepOnError;
 
+    protected $queueName;
     protected $queueOptions;
     protected $exchangeOptions;
 
@@ -32,17 +28,20 @@ class RabbitMQQueue extends Queue implements QueueContract
      * @var AmqpContext
      */
     private $context;
-    private $retryAfter;
     private $correlationId;
 
     public function __construct(AmqpContext $context, array $config)
     {
         $this->context = $context;
 
+        $this->queueName = $config['queue'] ?? $config['options']['queue']['name'];
         $this->queueOptions = $config['options']['queue'];
-        $this->queueOptions['arguments'] = isset($this->queueOptions['arguments']) ? json_decode($this->queueOptions['arguments'], true) : [];
+        $this->queueOptions['arguments'] = isset($this->queueOptions['arguments']) ?
+            json_decode($this->queueOptions['arguments'], true) : [];
 
         $this->exchangeOptions = $config['options']['exchange'];
+        $this->exchangeOptions['arguments'] = isset($this->exchangeOptions['arguments']) ?
+            json_decode($this->exchangeOptions['arguments'], true) : [];
 
         $this->sleepOnError = $config['sleep_on_error'] ?? 5;
     }
@@ -77,9 +76,17 @@ class RabbitMQQueue extends Queue implements QueueContract
             $message->setCorrelationId($this->getCorrelationId());
             $message->setContentType('application/json');
             $message->setDeliveryMode(AmqpMessage::DELIVERY_MODE_PERSISTENT);
+            
+            if (isset($options['headers'])) {
+                $message->setHeaders($options['headers']);
+            }
+            
+            if (isset($options['properties'])) {
+                $message->setProperties($options['properties']);
+            }
 
-            if ($this->retryAfter !== null) {
-                $message->setProperty(self::ATTEMPT_COUNT_HEADERS_KEY, $this->retryAfter);
+            if (isset($options['attempts'])) {
+                $message->setProperty(RabbitMQJob::ATTEMPT_COUNT_HEADERS_KEY, $options['attempts']);
             }
 
             $producer = $this->context->createProducer();
@@ -103,6 +110,24 @@ class RabbitMQQueue extends Queue implements QueueContract
         return $this->pushRaw($this->createPayload($job, $data), $queue, ['delay' => $this->secondsUntil($delay)]);
     }
 
+    /**
+     * Release a reserved job back onto the queue.
+     *
+     * @param  \DateTimeInterface|\DateInterval|int  $delay
+     * @param  string|object  $job
+     * @param  mixed   $data
+     * @param  string  $queue
+     * @param  int  $attempts
+     * @return mixed
+     */
+    public function release($delay, $job, $data, $queue, $attempts = 0)
+    {
+        return $this->pushRaw($this->createPayload($job, $data), $queue, [
+            'delay' => $this->secondsUntil($delay),
+            'attempts' => $attempts
+        ]);
+    }
+
     /** @inheritdoc */
     public function pop($queueName = null)
     {
@@ -122,18 +147,6 @@ class RabbitMQQueue extends Queue implements QueueContract
         return null;
     }
 
-    /**
-     * Sets the attempts member variable to be used in message generation.
-     *
-     * @param int $count
-     *
-     * @return void
-     */
-    public function setAttempts(int $count)
-    {
-        $this->retryAfter = $count;
-    }
-
     /**
      * Retrieves the correlation id, or a unique id.
      *
@@ -171,11 +184,12 @@ class RabbitMQQueue extends Queue implements QueueContract
      */
     private function declareEverything(string $queueName = null): array
     {
-        $queueName = $queueName ?: $this->queueOptions['name'];
+        $queueName = $queueName ?: $this->queueName;
         $exchangeName = $this->exchangeOptions['name'] ?: $queueName;
 
         $topic = $this->context->createTopic($exchangeName);
         $topic->setType($this->exchangeOptions['type']);
+        $topic->setArguments($this->exchangeOptions['arguments']);
         if ($this->exchangeOptions['passive']) {
             $topic->addFlag(AmqpTopic::FLAG_PASSIVE);
         }
